@@ -124,13 +124,13 @@ class EventsFrame(object):
 
     # Default standard column values
     default_keys = ['RID', 'YEAR', 'KEY']
-    default_beg =  ['BMP', 'BEG', 'FROM']
+    default_beg =  ['BMP', 'BEG', 'FROM', 'LOC']
     default_end =  ['EMP', 'END', 'TO']
     default_geom = ['geometry']
 
     def __init__(
         self, df, keys=None, beg=None, end=None, geom=None, route=None, 
-        closed='left_mod', sort=False, **kwargs):
+        closed=None, sort=False, **kwargs):
         # Log input values
         super(EventsFrame, self).__init__()
         self._df = df
@@ -186,6 +186,12 @@ class EventsFrame(object):
             # Retrieve default geometry column if df is geodataframe
             if isinstance(df, gpd.GeoDataFrame) and self.geom is None:
                 self.geom = df.geometry.name
+
+            # Reset log
+            try:
+                self.log.reset()
+            except:
+                pass
         else:
             raise TypeError("Input dataframe must be pandas DataFrame class "
                 "instance.")
@@ -380,6 +386,14 @@ class EventsFrame(object):
         self._geom = geom
         self._geom_loc = self.columns.index(geom) if not geom is None else None
 
+    @property
+    def is_point(self):
+        """
+        Returns True if the collection's beg and end columns are the same, 
+        implying that it is a collection of point events.
+        """
+        return self._beg == self._end
+
     @property
     def route(self):
         return self._route
@@ -557,30 +571,38 @@ class EventsFrame(object):
         else:
             return copy.copy(self)
     
-    def set_closed(self, closed, inplace=False):
+    def set_closed(self, closed=None, inplace=False):
         """
         Change whether ranges are closed on left, right, both, or neither side. 
         
         Parameters
         ----------
         closed : str {'left', 'left_mod', 'right', 'right_mod', 'both', 
-                'neither'}
+                'neither'}, optional
             Whether intervals are closed on the left-side, right-side, both or 
-            neither.
+            neither. If None, will default to 'left_mod' for linear events and 
+            'both' for point events.
         inplace : boolean, default False
             Whether to perform the operation in place on the parent range
             collection, returning None.
         """
-        if closed in RangeCollection._ops_closed:
-            if inplace:
-                self._closed = closed
+        # Ensure valid option selected
+        if closed is None:
+            if self.is_point:
+                closed = 'both'
             else:
-                rc = self.copy()
-                rc._closed = closed
-                return rc
-        else:
-            raise ValueError(f"Closed parameter must be one of "
+                closed = 'left_mod'
+        elif not closed in RangeCollection._ops_closed:
+            raise ValueError(
+                "Closed parameter must be one of "
                 f"{RangeCollection._ops_closed}.")
+        # Apply parameter
+        if inplace:
+            self._closed = closed
+        else:
+            ec = self.copy()
+            ec._closed = closed
+            return ec
 
     def geometry_from_xy(self, x, y, col_name='geometry', crs=None, 
             inplace=False):
@@ -1000,37 +1022,30 @@ class EventsFrame(object):
             **kwargs
         )
 
-    def to_windows(self, dissolve=False, **kwargs):
+    def to_grid(self, dissolve=False, **kwargs):
         """
-        Use the events dataframe to create sliding window events of a fixed 
-        length and a fixed number of steps, and which fill the bounds of each 
-        event.
+        Use the events dataframe to create a grid of zero-length, equidistant 
+        point events which span the bounds of each event.
 
         Parameters
         ----------
         length : numerical, default 1.0
-            A fixed length for all windows being defined.
-        steps : int, default 10
-            A number of steps per window length. The resulting step length will 
-            be equal to length / steps. For non-overlapped windows, use a steps 
-            value of 1.
-        fill : {'none','cut','left','right','extend','balance'}, default 'cut'
+            A fixed distance between each point on the grid.
+        fill : {'none','cut','extend','right','balance'}, default 'cut'
             How to fill a gap at the end of an event's range.
 
             Options
             -------
-            none : no window will be generated to fill the gap at the end of 
-                the input range.
-            cut : a truncated window will be created to fill the gap with a 
-                length less than the full window length.
-            left : the final window will be anchored on the end of the event  
-                and will extend the full window length to the left. 
-            right : the final window will be anchored on the grid defined by 
-                the step value, extending the full window length to the right, 
-                beyond the event's end value.
-            extend : the final window will be anchored on the grid defined by 
-                the step value, extending beyond the window length to the right
-                bound of the event.
+            none : no point will be generated at the end of the input range 
+                unless it falls directly on the defined grid distance.
+            cut : a point will be generated at the very end of the input range, 
+                at a distance less than or equal to the defined grid distance.
+            right : the final point will be generated at a distance equal to 
+                the defined grid distance, even if this extends beyond the full 
+                input range.
+            extend : a point will be generated at the very end of the input 
+                range, at a distance greater than or equal to the defined grid 
+                distance.
             balance : if the final range is greater than or equal to half the 
                 target range length, perform the cut method; if it is less, 
                 perform the extend method.
@@ -1049,10 +1064,98 @@ class EventsFrame(object):
             events[self.keys + [self.beg, self.end]].values,
             events.index.values
         )
+        grid = []
+        for (*keys, beg, end), index in gen:
+            # Build grid points
+            rng = RangeCollection.from_steps(beg, end, **kwargs).cut(beg, end)
+            locs = np.append(rng.begs, rng.ends[-1])
+            num_locs = len(locs)
+            # Assemble sliding window data
+            grid.append(
+                np.concatenate(
+                    [
+                        [keys]*num_locs,        # Event keys
+                        np.tile(locs, (2,1)).T, # Point locations
+                        [[index]]*num_locs      # Parent index value
+                    ],
+                    axis=1
+                )
+            )
+
+        # Merge and prepare data, return
+        grid = np.concatenate(grid, axis=0)
+        df = pd.DataFrame(
+            data=grid,
+            columns=self.keys + [self.beg, self.end, 'index_parent'],
+            index=None,
+        )
+        # Enforce data types
+        dtypes = {
+            **events.dtypes,
+            'index_parent': events.index.dtype
+        }
+        dtypes = {col: dtypes[col] for col in df.columns}
+        df = df.astype(dtypes, copy=False)
+        res = self.__class__(df, keys=self.keys, beg=self.beg, end=self.end)
+        return res
+
+    def to_windows(self, dissolve=False, endpoint=False, **kwargs):
+        """
+        Use the events dataframe to create sliding window events of a fixed 
+        length and a fixed number of steps, and which fill the bounds of each 
+        event.
+
+        Parameters
+        ----------
+        length : numerical, default 1.0
+            A fixed length for all windows being defined.
+        steps : int, default 1
+            A number of steps per window length. The resulting step length will 
+            be equal to length / steps. For non-overlapped windows, use a steps 
+            value of 1.
+        fill : {'none','cut','extend','left','right','balance'}, default 'cut'
+            How to fill a gap at the end of an event's range.
+
+            Options
+            -------
+            none : no window will be generated to fill the gap at the end of 
+                the input range.
+            cut : a truncated window will be created to fill the gap with a 
+                length less than the full window length.
+            extend : the final window will be anchored on the grid defined by 
+                the step value, extending beyond the window length to the right
+                bound of the event.
+            left : the final window will be anchored on the end of the input 
+                range and will extend the full window length to the left. 
+            right : the final window will be anchored on the grid defined by 
+                the step value, extending the full window length to the right, 
+                beyond the event's end value.
+            balance : if the final range is greater than or equal to half the 
+                target range length, perform the cut method; if it is less, 
+                perform the extend method.
+        
+        dissolve : bool, default False
+            Whether to dissolve the events dataframe before performing the 
+            transformation.
+        endpoint : bool, default False
+            Add a point event at the end of each event range.
+        """
+        # Dissolve events
+        if dissolve:
+            events = self.dissolve().df
+        else:
+            events = self.df
+        # Iterate over roads and create sliding window segments
+        gen = zip(
+            events[self.keys + [self.beg, self.end]].values,
+            events.index.values
+        )
         windows = []
         for (*keys, beg, end), index in gen:
             # Build sliding window ranges
             rng = RangeCollection.from_steps(beg, end, **kwargs).cut(beg, end)
+            if endpoint:
+                rng = rng.append(end, end)
             # Assemble sliding window data
             windows.append(
                 np.concatenate(
@@ -1158,9 +1261,10 @@ class EventsGroup(EventsFrame):
         geometry associated with each event if available. If provided, 
         certain additional class functionalities will be made available.
     closed : str {'left', 'left_mod', 'right', 'right_mod', 'both', 
-            'neither'}, default 'left_mod'
+            'neither'}, optional
         Whether intervals are closed on the left-side, right-side, both or 
-        neither.
+        neither. If None, will default to 'left_mod' for linear events and 
+        'both' for point events.
 
         Options
         -------
@@ -1174,9 +1278,10 @@ class EventsGroup(EventsFrame):
             the left when the previous range is not consecutive.
         both : ranges are always closed on both sides
         neither : ranges are never closed on either side
+
     """
 
-    def __init__(self, df, beg=None, end=None, geom=None, closed='left_mod', 
+    def __init__(self, df, beg=None, end=None, geom=None, closed=None, 
         **kwargs):
         # Initialize EventsFrame superclass
         super(EventsGroup, self).__init__(
@@ -1211,9 +1316,16 @@ class EventsGroup(EventsFrame):
     @property
     def lengths(self):
         """
-        Lengths of all events ranges.
+        Lengths of all event ranges.
         """
         return self.rng.lengths
+
+    @property
+    def centers(self):
+        """
+        Centers of all event ranges.
+        """
+        return self.rng.centers
             
     @property
     def shape(self):
@@ -1221,8 +1333,8 @@ class EventsGroup(EventsFrame):
     
     def _build_rng(self):
         # Build range collection
-        rng = RangeCollection.from_array(self.df[[self.beg,self.end]].values.T,
-                                         closed=self.closed, sort=False)
+        rng = RangeCollection.from_array(
+            self.df[[self.beg,self.end]].values, closed=self.closed, sort=False)
         self.rng = rng
 
     def set_closed(self, closed, inplace=False):
@@ -1259,59 +1371,75 @@ class EventsGroup(EventsFrame):
         except AttributeError:
             pass
 
-    def intersecting(self, beg, end=None, closed=None, mask=False, **kwargs):
+    def intersecting(self, beg=None, end=None, other=None, closed='both', 
+        get_mask=False, **kwargs):
         """
         Retrieve a selection of records from the group of events based 
         on provided begin and end locations.
 
         Parameters
         ----------
-        beg : float
-            Beginning milepost of the overlaid segment.
-        end : float, optional
-            Ending milepost of the overlaid segment. If not provided, a point 
-            overlay is assumed.
-        closed : str {'left', 'left_mod', 'right', 'right_mod', 'both', 
-                'neither'}, default 'left'
-            Whether intervals are closed on the left-side, right-side, both or 
-            neither.
+        beg, end : numerical or array-like, optional
+            The begin and end locations of the range or ranges to be tested. If 
+            a single range is to be tested, provide a numeric value. If 
+            multiple, provide an array-like with a single begin and end value 
+            for each range. If no end parameter provided, point locations will 
+            be assumed and end will be set equal to beg. Not required if other 
+            parameter is used.
+        other : EventsGroup, optional
+            Other EventsGroup instance to be intersected with this one. Can 
+            be provided instead of beg, end, and closed parameters and will 
+            take precedence over other input.
+        closed : str {'left', 'right', 'both', 'neither'}, default 'both'
+            Whether input interval is closed on the left-side, right-side, both 
+            or neither.
 
             Options
             -------
             left : ranges are always closed on the left and never closed on the 
                 right.
-            left_mod : ranges are always closed on the left and only closed on 
-                the right when the next range is not consecutive.
             right : ranges are always closed on the right and never closed on 
                 the right.
-            right_mod : ranges are always closed on the right and only closed 
-                on the left when the previous range is not consecutive.
             both : ranges are always closed on both sides
             neither : ranges are never closed on either side
 
-        mask : bool, default False
-            Whther to return a boolean mask for selecting from the events 
+        get_mask : bool, default False
+            Whether to return a boolean mask for selecting from the events 
             dataframe instead of the selection from the dataframe itself.
         """
+        # Deprecation
+        get_mask = kwargs.get('mask', get_mask)
+        # Check for other input
+        if not other is None:
+            if not isinstance(other, EventsGroup):
+                raise TypeError(
+                    "If provided, input other parameter must be valid "
+                    "EventsGroup instance.")
+            other = other.rng
         # Intersect range
-        mask_ = self.rng.intersecting(beg=beg, end=end, closed=closed, **kwargs)
-        if mask:
-            return mask_
+        mask = self.rng.intersecting(
+            beg=beg, end=end, other=other, closed=closed, **kwargs)
+        if get_mask:
+            return mask
         else:
-            df = self.df.loc[mask_, :]
+            if mask.ndim > 1:
+                mask = mask.any(axis=1)
+            df = self.df.loc[mask, :]
             return df
 
-    def overlay(self, beg=None, end=None, arr=False, **kwargs):
+    def overlay(self, beg=None, end=None, other=None, **kwargs):
         """
         Compute overlap of the input bounds with respect to the 
         events group.
         
         Parameters
         ----------
-        beg : float
-            Beginning milepost of the overlaid segment.
-        end : float
-            Ending milepost of the overlaid segment.
+        beg, end : scalar or array of scalars
+            Begin and end locations of the overlaid range(s).
+        other : EventsGroup, optional
+            Other EventsGroup instance to be overlaid with this one. Can be 
+            provided instead of beg and end parameters and will take precedence 
+            over other input.
         normalize : boolean, default True
             Whether overlapping lengths should be normalized range length to 
             give a proportional result.
@@ -1333,17 +1461,18 @@ class EventsGroup(EventsFrame):
             (denominator) is equal to zero, e.g., when the overlay range has a 
             length of zero and how='right'. If not provided, all instances of 
             zero division will return float value 0.0.
-        arr : bool, default False
-            Whther to return an array of resulting overlay values instead of 
-            a pandas Series with an index matching the events dataframe.
         """
+        # Check for other input
+        if not other is None:
+            if not isinstance(other, EventsGroup):
+                raise TypeError(
+                    "If provided, input other parameter must be valid "
+                    "EventsGroup instance.")
+            beg = other.rng.begs
+            end = other.rng.ends
         # Compute range overlaps
         weights = self.rng.overlay(beg=beg, end=end, **kwargs)
-        if arr:
-            return weights
-        else:
-            sr = pd.Series(data=weights, index=self.df.index)
-            return sr
+        return weights
     
     def overlay_average(self, beg=None, end=None, cols=None, weighted=True, 
                         zeroweight=None, how='right', weights=None, 
@@ -1582,9 +1711,10 @@ class EventsCollection(EventsFrame):
         geometry associated with each event if available. If provided, 
         certain additional class functionalities will be made available.
     closed : str {'left', 'left_mod', 'right', 'right_mod', 'both', 
-            'neither'}, default 'left_mod'
+            'neither'}, optional
         Whether intervals are closed on the left-side, right-side, both or 
-        neither.
+        neither. If None, will default to 'left_mod' for linear events and 
+        'both' for point events.
 
         Options
         -------
@@ -1605,7 +1735,7 @@ class EventsCollection(EventsFrame):
     """
 
     def __init__(self, df, keys=None, beg=None, end=None, 
-        geom=None, closed='left_mod', sort=False, **kwargs):
+        geom=None, closed=None, sort=False, **kwargs):
         # Validate keys option
         if keys is None:
             raise Exception("If no keys are required to define unique groups "
@@ -1827,7 +1957,7 @@ class EventsCollection(EventsFrame):
         # Return retrieved column data
         return res
 
-    def merge(self, other, closed=None):
+    def merge(self, other):
         """
         Create an EventsMerge instance with this collection as the left and the 
         other collection as the right. This can then be used to retrieve 
@@ -1841,11 +1971,6 @@ class EventsCollection(EventsFrame):
             with this events collection, producing an EventsMerge instance 
             which can be used to perform various overlay operations to retrieve 
             attributes and more from the target collection.
-        closed : str {'left', 'left_mod', 'right', 'right_mod', 'both', 
-                'neither'}, optional
-            Whether intervals on the target events frame are closed on the 
-            left-side, right-side, both or neither. If not provided, will 
-            default to the already-set parameter on the events frame.
         """
         # Create merge
         em = EventsMerge(self, other)
@@ -1944,8 +2069,8 @@ class EventsCollection(EventsFrame):
                     if empty:
                         dfs.append(self._build_empty())
                     else:
-                        raise KeyError("Invalid EventsCollection keys: "
-                                       f"{keys_i}")
+                        raise KeyError(
+                            f"Invalid EventsCollection keys: {keys_i}")
         # Log and return retrieved group
         if ndim == 1:
             return self._build_group(dfs[0])
@@ -2005,8 +2130,13 @@ class EventsCollection(EventsFrame):
         
         # Produce filtered collection
         df = self.df.loc[mask, :]
-        ec = EventsCollection(df, keys=new_keys, beg=self.beg, end=self.end, 
-                              geom=self.geom, closed=self.closed)
+        try:
+            ec = EventsCollection(
+                df, keys=new_keys, beg=self.beg, end=self.end, 
+                geom=self.geom, closed=self.closed)
+        except:
+            display(df)
+            raise ValueError()
         return ec
 
     def get_matching(self, other, **kwargs):
@@ -2029,8 +2159,13 @@ class EventsCollection(EventsFrame):
         the events collection's dataframe.
         """
         # Build and return events group
-        return EventsGroup(df=df, beg=self.beg, end=self.end, 
-                           geom=self.geom, closed=self.closed)
+        try:
+            return EventsGroup(
+                df=df, beg=self.beg, end=self.end, geom=self.geom, 
+                closed=self.closed)
+        except:
+            display(df)
+            raise ValueError()
 
 
 ####################
