@@ -19,6 +19,7 @@ Modified:
 ################
 
 
+from xml.dom.minidom import Attr
 import numpy as np
 from shapely.geometry import LineString, MultiLineString
 import shapely
@@ -74,7 +75,10 @@ class MLSRoute(object):
         return self.rte_length
     
     def __str__(self):
-        return 'MLSRoute('+str(self.mls)+')'
+        return self.wkt
+
+    def __repr__(self):
+        return self.wkt
         
     @property
     def mls(self):
@@ -92,6 +96,7 @@ class MLSRoute(object):
                     "LineString.")
         # Log geometry
         self._mls = obj
+        self._wkt = None
         # Accumulate vector lengths to generate geometry breaks
         self._mls_breaks = \
             np.concatenate([[0]] + self.element_lengths).cumsum()
@@ -104,6 +109,15 @@ class MLSRoute(object):
             pass
         self._mls_ranges = rc
 
+    @property
+    def wkt(self):
+        try:
+            assert self._wkt is not None
+            return self._wkt
+        except AssertionError:
+            self._wkt = self.to_wkt()
+            return self._wkt
+
     @property
     def mls_breaks(self):
         return self._mls_breaks
@@ -127,7 +141,8 @@ class MLSRoute(object):
             # Coerce np.ndarray
             data = [np.asarray(x) for x in data]
             # Check data length
-            assert all(len(r)==len(l.coords) for r, l in zip(data, self._mls))
+            assert all(len(r)==len(l.coords) for \
+                r, l in zip(data, self._mls.geoms))
         except:
             raise ValueError(
                 "Route breaks data should be list of array-like of numeric "
@@ -160,7 +175,7 @@ class MLSRoute(object):
         try:
             return self._num_lines
         except AttributeError:
-            self._num_lines = len(self.mls)
+            self._num_lines = len(self.mls.geoms)
             return self._num_lines
     
     @property
@@ -290,7 +305,7 @@ class MLSRoute(object):
             full_lines = combine_mpgs(full_lines, cls=MultiLineString)
             ranges = np.stack([begs_new, ends_new]).T
         # - One begin/end point per LineString provided
-        elif len(begs) == len(ends) == sum(len(i) for i in full_lines):
+        elif len(begs) == len(ends) == sum(len(i.geoms) for i in full_lines):
             ranges = np.stack([begs, ends]).T
             full_lines = combine_mpgs(full_lines, cls=MultiLineString)
         # - Invalid input type
@@ -299,11 +314,61 @@ class MLSRoute(object):
                 "Must provide a number of begin and end mile post values "
                 "equal to the number of lines if providing multiple values. "
                 f"Provided: {len(begs):,.0f} begs, {len(ends):,.0f} ends, "
-                f"{len(mls):,.0f} lines.")
+                f"{len(mls.geoms):,.0f} lines.")
         
         # Return generated MLSRoute instance based on input parameters
         return cls(full_lines, rte_ranges=ranges, **kwargs)
 
+    @classmethod
+    def from_wkt(cls, wkt, **kwargs):
+        """
+        Create an MLSRoute instance from a WKT string for a MULTILINESTRING or 
+        LINESTRING with three to four dimensions, with the last dimension 
+        being interpreted as M values.
+        
+        Parameters
+        ----------
+        wkt : str
+            WKT string representing a MULTILINESTRING or LINESTRING with three 
+            to four dimensions, with the last dimension representing the 
+            geometry's M values.
+        **kwargs
+            Keyword arguments to be input in the MLSRoute constructor.
+        """
+        # Simplify text and validate content
+        if 'MULTILINESTRING' in wkt:
+            simplified = wkt.split('((')[1] \
+                .replace('),',';').replace('(','').replace(')','')
+        elif 'LINESTRING' in wkt:
+            simplified = wkt.split('(')[1].replace(')','')
+        else:
+            raise ValueError(
+                "Provided WKT must represent multilinestring or linestring "
+                "data.")
+        # Iterate through groups of coordinates to parse M values
+        data = []
+        breaks = []
+        for coord_group in simplified.split(';'):
+            # Iterate through individual coordinates in the group
+            data_group = []
+            breaks_group = []
+            for coord in coord_group.split(','):
+                try:
+                    coords = [float(x) for x in coord.strip().split(' ')]
+                    dims = len(coords)
+                    assert (dims > 2) & (dims < 5)
+                except AssertionError:
+                    raise ValueError(
+                        "Input WKT must have between three and four "
+                        "dimensions.")
+                data_group.append(coords[:-1])
+                breaks_group.append(coords[-1])
+            data.append(data_group)
+            breaks.append(breaks_group)
+
+        # Create MLSRoute
+        return MLSRoute(MultiLineString(data), rte_breaks=breaks, **kwargs)
+
     @classmethod
     def concatenate(cls, routes, **kwargs):
         """
@@ -340,7 +405,7 @@ class MLSRoute(object):
         """
         # Store all vectors
         lengths = []
-        for line in self.mls:
+        for line in self.mls.geoms:
             # Log vector length
             coords = line.coords
             lengths.append(np.asarray([LineString(coords[i-1:i+1]).length \
@@ -357,7 +422,7 @@ class MLSRoute(object):
             # Coerce as numpy array, check shape
             try:
                 data = np.asarray(data)
-                assert data.shape == (len(self._mls), 2)
+                assert data.shape == (len(self._mls.geoms), 2)
             except:
                 raise ValueError(
                     "Input ranges should be array-like of numeric values with "
@@ -377,6 +442,45 @@ class MLSRoute(object):
             lengths[-1] = rng[-1] # Snap last value to range bound
             all_breaks.append(np.concatenate([rng[0], lengths], axis=None))
         return all_breaks
+
+    def to_wkt(self, decimals=None):
+        """
+        Produce a WKT string representing the object with the underlying 
+        MultiLineString appended with M values represented in the rte_breaks 
+        property.
+        """
+        # Define number formatter
+        if decimals is None:
+            fmt = lambda x: str(x)
+        elif isinstance(decimals, int):
+            fmt = lambda x: '{x:.{decimals:.0f}f}' \
+                .format(decimals=decimals, x=float(x))
+        else:
+            raise ValueError("Decimals parameter must be an integer.")
+
+        # Retrieve the base WKT for the multilinestring
+        wkt = self.mls.wkt
+        simplified = \
+            wkt.split('((')[1].replace('),',';').replace('(','').replace(')','')
+        # Iterate through groups of coordinates to append M values
+        data = []
+        zipped = zip(simplified.split(';'), self.rte_breaks)
+        for coord_group, m_group in zipped:
+            # Iterate through individual coordinates in the group
+            data_group = []
+            for coord, m in zip(coord_group.split(','), m_group):
+                point = ' '.join(fmt(x) for x in coord.strip().split(' ') + [m])
+                data_group.append(point)
+            data.append('(' + ', '.join(data_group) + ')')
+
+        # Determine WKT prefix
+        prefix = wkt.split(' (')[0]
+        if prefix[-1] == 'Z':
+            prefix += 'M '
+        else:
+            prefix += ' M '
+        # Return combined WKT string
+        return prefix + '(' + ', '.join(data) + ')'
     
     def copy(self, deep=False):
         """
@@ -710,7 +814,7 @@ class MLSRoute(object):
         breaks = []
         
         # Iterate over the LineStrings in the MLS
-        for num, (line, breaks_all) in enumerate(zip(self.mls,
+        for num, (line, breaks_all) in enumerate(zip(self.mls.geoms,
                  self.rte_breaks)):
             
             # Get the number of ranges in the LineString
@@ -904,7 +1008,7 @@ def combine_mpgs(objs, cls=None):
     new = []
     for obj in objs:
         if isinstance(obj, shapely.geometry.base.BaseMultipartGeometry):
-            new.extend(list(obj))
+            new.extend(list(obj.geoms))
         elif isinstance(obj, shapely.geometry.base.BaseGeometry):
             new.extend([obj])
         else:
@@ -922,7 +1026,7 @@ def _distribute_dimensions(mls, beg, end):
         raise ValueError("Input MLS must be MultiLineString type.")
     # Compute dimensions
     delta = end - beg
-    lengths = np.array([ls.length for ls in mls])
+    lengths = np.array([ls.length for ls in mls.geoms])
     proportions = np.cumsum(lengths / lengths.sum() * delta)
     begs = np.insert(proportions[:-1] + beg, 0, beg)
     ends = proportions + beg
